@@ -11,7 +11,7 @@
 #
 # Credits:  (see the CREDITS file)
 #
-# Copyright (C) 1999-2007 Michael Rash (mbr@cipherdyne.org)
+# Copyright (C) 1999-2008 Michael Rash (mbr@cipherdyne.org)
 #
 # License (GNU Public License):
 #
@@ -29,7 +29,7 @@
 #
 #########################################################################
 #
-# $Id: install.pl 2055 2007-05-25 03:09:44Z mbr $
+# $Id: install.pl 2178 2008-06-07 15:21:22Z mbr $
 #
 
 use Cwd;
@@ -41,21 +41,20 @@ use Getopt::Long;
 use strict;
 
 #============== config ===============
-my $INIT_DIR     = '/etc/init.d';
 my $USRSBIN_DIR  = '/usr/sbin';  ### consistent with FHS (Filesystem
                                  ### Hierarchy Standard)
 my $USRBIN_DIR   = '/usr/bin';   ### consistent with FHS
-my $RUNLEVEL;    ### This should only be set if install.pl
-                 ### cannot determine the correct runlevel
 
 my $psad_conf_file  = 'psad.conf';
 
 ### system binaries ###
 my $chkconfigCmd = '/sbin/chkconfig';
 my $rcupdateCmd  = '/sbin/rc-update';  ### Gentoo
+my $updatercdCmd = '/usr/sbin/update-rc.d';  ### Ubuntu
 my $makeCmd      = '/usr/bin/make';
 my $perlCmd      = '/usr/bin/perl';
 my $wgetCmd      = '/usr/bin/wget';
+my $runlevelCmd  = '/sbin/runlevel';
 #============ end config ============
 
 my %file_vars = (
@@ -91,6 +90,10 @@ my %required_perl_modules = (
         'force-install' => 0,
         'mod-dir' => 'Bit-Vector'
     },
+    'Storable', => {
+        'force-install' => 0,
+        'mod-dir' => 'Storable'
+    },
     'Date::Calc', => {
         'force-install' => 0,
         'mod-dir' => 'Date-Calc'
@@ -147,6 +150,12 @@ my $force_mod_re = '';
 my $exclude_mod_re = '';
 my $no_rm_old_lib_dir = 0;
 my $syslog_conf = '';
+my $locale = 'C';  ### default LC_ALL env variable
+my $no_locale = 0;
+my $init_dir = '/etc/init.d';
+my $init_name = 'psad';
+my $install_syslog_fifo = 0;
+my $runlevel = -1;
 
 ### make Getopts case sensitive
 Getopt::Long::Configure('no_ignore_case');
@@ -163,10 +172,19 @@ Getopt::Long::Configure('no_ignore_case');
     'syslog-conf=s'     => \$syslog_conf, ### specify path to syslog config file.
     'no-syslog-test'    => \$skip_syslog_test,
     'uninstall'         => \$uninstall,   ### Uninstall psad.
+    'init-dir=s'        => \$init_dir,
+    'init-name=s'       => \$init_name,
+    'install-syslog-fifo' => \$install_syslog_fifo,
+    'runlevel=i'        => \$runlevel,
+    'LC_ALL=s'          => \$locale,
+    'no-LC_ALL'         => \$no_locale,
     'help'              => \$help         ### Display help.
 ));
 &usage(0) if $help;
 
+### set LC_ALL env variable
+$ENV{'LC_ALL'} = $locale unless $no_locale;
+
 ### import paths from default psad.conf
 &import_config();
 
@@ -175,8 +193,9 @@ my @LOGR_FILES   = (*STDOUT, $config{'INSTALL_LOG_FILE'});
 $force_mod_re = qr|$force_mod_re| if $force_mod_re;
 $exclude_mod_re = qr|$exclude_mod_re| if $exclude_mod_re;
 
-$cmds{'make'} = $makeCmd;
-$cmds{'perl'} = $perlCmd;
+$cmds{'make'}     = $makeCmd;
+$cmds{'perl'}     = $perlCmd;
+$cmds{'runlevel'} = $runlevelCmd;
 
 if (-e $config{'INSTALL_LOG_FILE'}) {
     open INSTALL, "> $config{'INSTALL_LOG_FILE'}" or
@@ -192,19 +211,20 @@ if ($distro eq 'redhat' or $distro eq 'fedora') {
 } elsif ($distro eq 'gentoo') {
     ### add rc-update if we are running on a gentoo distro
     $cmds{'rc-update'} = $rcupdateCmd;
+} elsif ($distro eq 'ubuntu') {
+    ### add update-rc.d if we are running on an ubuntu distro
+    $cmds{'update-rc.d'} = $updatercdCmd;
 }
 
-my $init_dir = '';
-
-if (-d $INIT_DIR) {
-    $init_dir = $INIT_DIR;
-} elsif (-d '/etc/rc.d/init.d') {
-    $init_dir = '/etc/rc.d/init.d';
-} elsif (-d '/etc/rc.d') {  ### for Slackware
-    $init_dir = '/etc/rc.d';
-} else {
-    die "[*] Cannot find the init script directory, edit ",
-        "the \$INIT_DIR variable.\n";
+unless (-d $init_dir) {
+    if (-d '/etc/rc.d/init.d') {
+        $init_dir = '/etc/rc.d/init.d';
+    } elsif (-d '/etc/rc.d') {  ### for Slackware
+        $init_dir = '/etc/rc.d';
+    } else {
+        die "[*] Cannot find the init script directory, use ",
+            "--init-dir <path>\n";
+    }
 }
 
 ### need to make sure this exists before attempting to
@@ -432,8 +452,17 @@ sub install() {
         mkdir $config{'PSAD_CONF_DIR'},0500;
     }
 
-    ### get syslog daemon (e.g. syslog, syslog-ng, or metalog)
-    my $syslog_str = &query_syslog();
+    my $syslog_str = '';
+
+    if ($install_syslog_fifo) {
+        ### get syslog daemon (e.g. syslog, rsyslog syslog-ng, or metalog)
+        $syslog_str = &query_syslog();
+    } else {
+        print
+"[+] psad by default parses iptables log messages from the /var/log/messages\n",
+"    file, but you can alter this with the IPT_SYSLOG_FILE variable in the\n",
+"    /etc/psad/psad.conf file.\n";
+    }
 
     my $preserve_rv = 0;
     if (-e "$config{'PSAD_CONF_DIR'}/psad.conf") {
@@ -554,82 +583,98 @@ sub install() {
         &set_hostname($file);
     }
 
-    &put_string('SYSLOG_DAEMON', $syslog_str,
-        "$config{'PSAD_CONF_DIR'}/psad.conf");
+    if ($install_syslog_fifo) {
+        &put_string('SYSLOG_DAEMON', $syslog_str,
+            "$config{'PSAD_CONF_DIR'}/psad.conf");
 
-    if ($syslog_str ne 'ulogd') {
-        my $restarted_syslog = 0;
-        if ($syslog_str eq 'syslogd') {
-            if (-e $syslog_conf) {
-                &append_fifo_syslog($syslog_conf);
-                if (((system "$cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
-                    &logr("[+] HUP signal sent to syslogd.\n");
-                    $restarted_syslog = 1;
-                }
-            }
-        } elsif ($syslog_str eq 'syslog-ng') {
-            if (-e $syslog_conf) {
-                &append_fifo_syslog_ng($syslog_conf);
-                if (((system "$cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
-                    &logr("[+] HUP signal sent to syslog-ng.\n");
-                    $restarted_syslog = 1;
-                }
-            }
-        } elsif ($syslog_str eq 'metalog') {
-            if (-e $syslog_conf) {
-                &config_metalog($syslog_conf);
-                &logr("[-] Metalog support is shaky in psad.  " .
-                    "Use at your own risk.\n");
-                ### don't send warning about not restarting metalog daemon
-                $restarted_syslog = 1;
-            }
-        }
-
-        unless ($restarted_syslog) {
-            &logr("[-] Could not restart any syslog daemons.\n");
-        }
-    }
-
-    if (-x $cmds{'iptables'} and not $skip_syslog_test) {
-        &logr("[+] Found iptables. Testing syslog configuration:\n");
-        ### make sure we actually see packets being logged by
-        ### the firewall.
         if ($syslog_str ne 'ulogd') {
-            if (&test_syslog_config($syslog_str)) {
-                &logr("[+] Successful $syslog_str reconfiguration.\n\n");
-            } else {
-                if (&query_init_script_restart_syslog()) {
+            my $restarted_syslog = 0;
+            if ($syslog_str eq 'syslogd') {
+                if (-e $syslog_conf) {
+                    &append_fifo_syslog($syslog_conf);
+                    if (((system "$cmds{'killall'} -HUP syslogd 2> /dev/null")>>8) == 0) {
+                        &logr("[+] HUP signal sent to syslogd.\n");
+                        $restarted_syslog = 1;
+                    }
+                }
+            } elsif ($syslog_str eq 'rsyslogd') {
+                if (-e $syslog_conf) {
+                    &append_fifo_syslog($syslog_conf);
+                    if (((system "$cmds{'killall'} -HUP rsyslogd 2> /dev/null")>>8) == 0) {
+                        &logr("[+] HUP signal sent to rsyslogd.\n");
+                        $restarted_syslog = 1;
+                    }
+                }
 
-                    my $restarted = 0;
-                    if ($syslog_str eq 'syslog-ng') {
-                        if (-e "$INIT_DIR/syslog-ng") {
-                            system "$INIT_DIR/syslog-ng restart";
-                            $restarted = 1;
+            } elsif ($syslog_str eq 'syslog-ng') {
+                if (-e $syslog_conf) {
+                    &append_fifo_syslog_ng($syslog_conf);
+                    if (((system "$cmds{'killall'} -HUP syslog-ng 2> /dev/null")>>8) == 0) {
+                        &logr("[+] HUP signal sent to syslog-ng.\n");
+                        $restarted_syslog = 1;
+                    }
+                }
+            } elsif ($syslog_str eq 'metalog') {
+                if (-e $syslog_conf) {
+                    &config_metalog($syslog_conf);
+                    &logr("[-] Metalog support is shaky in psad.  " .
+                        "Use at your own risk.\n");
+                    ### don't send warning about not restarting metalog daemon
+                    $restarted_syslog = 1;
+                }
+            }
+
+            unless ($restarted_syslog) {
+                &logr("[-] Could not restart any syslog daemons.\n");
+            }
+        }
+
+        if (-x $cmds{'iptables'} and not $skip_syslog_test) {
+            &logr("[+] Found iptables. Testing syslog configuration:\n");
+            ### make sure we actually see packets being logged by
+            ### the firewall.
+            if ($syslog_str ne 'ulogd') {
+                if (&test_syslog_config($syslog_str)) {
+                    &logr("[+] Successful $syslog_str reconfiguration.\n\n");
+                } else {
+                    if (&query_init_script_restart_syslog()) {
+
+                        my $restarted = 0;
+                        if ($syslog_str eq 'syslog-ng') {
+                            if (-e "$init_dir/syslog-ng") {
+                                system "$init_dir/syslog-ng restart";
+                                $restarted = 1;
+                            }
+                        } elsif ($syslog_str eq 'rsyslogd') {
+                            if (-e "$init_dir/sysklogd") {
+                                system "$init_dir/sysklogd restart";
+                                $restarted = 1;
+                            } elsif (-e "$init_dir/syslog") {
+                                system "$init_dir/syslog restart";
+                                $restarted = 1;
+                            }
+                        } else {
+                            if (-e "$init_dir/rsyslog") {
+                                system "$init_dir/rsyslog restart";
+                                $restarted = 1;
+                            }
+                        }
+                        ### test syslog config again now that we
+                        ### have restarted syslog via the init script
+                        ### instead of relying on a HUP signal to
+                        ### syslog
+                        if ($restarted) {
+                            if (&test_syslog_config($syslog_str)) {
+                                &logr("[+] Successful $syslog_str reconfiguration.\n\n");
+                            } else {
+                                &logr("[-] Unsuccessful $syslog_str reconfiguration.\n");
+                                &logr("    Consult the psad man page for the basic " .
+                                    "$syslog_str requirement to get psad to work.\n\n");
+                            }
                         }
                     } else {
-                        if (-e "$INIT_DIR/sysklogd") {
-                            system "$INIT_DIR/sysklogd restart";
-                            $restarted = 1;
-                        } elsif (-e "$INIT_DIR/syslog") {
-                            system "$INIT_DIR/syslog restart";
-                            $restarted = 1;
-                        }
+                        &logr("[-] Ok, hoping that psad can get packet data anyway.\n");
                     }
-                    ### test syslog config again now that we
-                    ### have restarted syslog via the init script
-                    ### instead of relying on a HUP signal to
-                    ### syslog
-                    if ($restarted) {
-                        if (&test_syslog_config($syslog_str)) {
-                            &logr("[+] Successful $syslog_str reconfiguration.\n\n");
-                        } else {
-                            &logr("[-] Unsuccessful $syslog_str reconfiguration.\n");
-                            &logr("    Consult the psad man page for the basic " .
-                                "$syslog_str requirement to get psad to work.\n\n");
-                        }
-                    }
-                } else {
-                    &logr("[-] Ok, hoping that psad can get packet data anyway.\n");
                 }
             }
         }
@@ -655,10 +700,10 @@ sub install() {
     }
 
     if ($init_dir) {
-        &logr("[+] Copying $init_file -> ${init_dir}/psad\n");
-        copy $init_file, "${init_dir}/psad" or die "[*] Could not copy ",
-            "$init_file -> ${init_dir}/psad: $!";
-        &perms_ownership("${init_dir}/psad", 0744);
+        &logr("[+] Copying $init_file -> ${init_dir}/$init_name\n");
+        copy $init_file, "${init_dir}/$init_name" or die "[*] Could not copy ",
+            "$init_file -> ${init_dir}/$init_name: $!";
+        &perms_ownership("${init_dir}/$init_name", 0744);
         &enable_psad_at_boot($distro);
     }
 
@@ -835,44 +880,11 @@ sub uninstall() {
             rmtree $dir;
         }
     }
-    my $running_syslogd = 0;
-    my $running_syslog_ng = 0;
-    print "[+] Restoring /etc/syslog.conf.orig -> /etc/syslog.conf\n";
-    if (-e '/etc/syslog.conf.orig') {
-        move '/etc/syslog.conf.orig', '/etc/syslog.conf' or die "[*] Could not ",
-            "move /etc/syslog.conf.orig -> /etc/syslog.conf: $!";
-        $running_syslogd = 1;
-    } elsif (-e '/etc/syslog.conf') {
-        print "[+] /etc/syslog.conf.orig does not exist. ",
-            " Editing /etc/syslog.conf directly.\n";
-        open ESYS, '< /etc/syslog.conf' or
-            die "[*] Unable to open /etc/syslog.conf: $!\n";
-        my @sys = <ESYS>;
-        close ESYS;
-        open CSYS, '> /etc/syslog.conf' or die "[*] Could not open ",
-            "/etc/syslog.conf: $!";
-        for my $line (@sys) {
-            chomp $line;
-            ### don't print the psadfifo line
-            print CSYS "$line\n" if ($line !~ /psadfifo/);
-        }
-        close CSYS;
-        $running_syslogd = 1;
-    } elsif (-e '/etc/syslog-ng/syslog-ng.conf.orig') {
-        move '/etc/syslog-ng/syslog-ng.conf.orig', '/etc/syslog-ng/syslog-ng.conf'
-            or die "[*] Could not move /etc/syslog.conf.orig ",
-                "-> /etc/syslog.conf: $!";
-        $running_syslog_ng = 1;
-    }
-    if ($running_syslogd) {
-        print "[+] Restarting syslog.\n";
-        system "$cmds{'killall'} -HUP syslogd";
-    } elsif ($running_syslog_ng) {
-        print "[+] Restarting syslog.\n";
-        system "$cmds{'killall'} -HUP syslog-ng";
-    }
-    print "\n";
-    print "[+] psad has been uninstalled!\n";
+    print "[+] You will need to remove psad-specific config lines ",
+        "from your syslog config.\n",
+        "    Your original syslog config at psad install time is preserved here:\n",
+        "    $config{'CONF_ARCHIVE_DIR'}\n";
+    print "\n[+] psad has been uninstalled!\n";
     return;
 }
 
@@ -963,7 +975,7 @@ sub install_perl_module() {
             print "[+] Module $mod_name is already installed in the ",
                 "system perl tree, skipping.\n";
         } else {
-            ### install the module in the /usr/lib/fwknop directory because
+            ### install the module in the /usr/lib/psad directory because
             ### it is not already installed.
             $install_module = 1;
         }
@@ -1606,6 +1618,7 @@ sub get_distro() {
             chomp $line;
             return 'redhat' if $line =~ /red\s*hat/i;
             return 'fedora' if $line =~ /fedora/i;
+            return 'ubuntu' if $line =~ /ubuntu/i;
         }
     }
     return 'NA';
@@ -1734,12 +1747,12 @@ sub query_email() {
 }
 
 sub query_syslog() {
-    &logr("\n[+] psad supports the syslogd, syslog-ng, ulogd, and\n" .
+    &logr("\n[+] psad supports the syslogd, rsyslogd, syslog-ng, ulogd, and\n" .
         "    metalog logging daemons.  Which system logger is running?\n\n");
     my $ans = '';
-    while ($ans ne 'syslogd' and $ans ne 'syslog-ng' and $ans ne 'ulogd'
-            and $ans ne 'metalog') {
-        &logr("    syslogd / syslog-ng / ulogd / metalog? [syslogd] ");
+    while ($ans ne 'syslogd' and $ans ne 'rsyslogd' and
+            $ans ne 'syslog-ng' and $ans ne 'ulogd' and $ans ne 'metalog') {
+        &logr("    syslogd / rsyslogd / syslog-ng / ulogd / metalog? [syslogd] ");
         $ans = <STDIN>;
         if ($ans eq "\n") {  ### allow default to take over
             $ans = 'syslogd';
@@ -1749,6 +1762,9 @@ sub query_syslog() {
         if ($ans eq 'syslogd') {
             ### allow command line --syslog-conf arg to take over
             $syslog_conf = '/etc/syslog.conf' unless $syslog_conf;
+        } elsif ($ans eq 'rsyslogd') {
+            ### allow command line --syslog-conf arg to take over
+            $syslog_conf = '/etc/rsyslog.conf' unless $syslog_conf;
         } elsif ($ans eq 'syslog-ng') {
             ### allow command line --syslog-conf arg to take over
             $syslog_conf = '/etc/syslog-ng/syslog-ng.conf' unless $syslog_conf;
@@ -1757,8 +1773,12 @@ sub query_syslog() {
             $syslog_conf = '/etc/metalog/metalog.conf' unless $syslog_conf;
         }
         if ($ans ne 'ulogd' and $syslog_conf and not -e $syslog_conf) {
+            if (-e '/etc/rsyslog.conf') {
+                warn "[-] It looks like /etc/rsyslog.conf exists, ",
+                    "did you mean rsyslog?\n";
+            }
             die
-"[-] The config file $syslog_conf does not exist. Re-run install.pl\n",
+"[*] The config file $syslog_conf does not exist. Re-run install.pl\n",
 "    with the --syslog-conf argument to specify the path to the syslog\n",
 "    daemon config file.\n";
         }
@@ -1766,6 +1786,7 @@ sub query_syslog() {
     die "[-] Invalid syslog daemon \"$ans\"\n"
         unless ($ans and
             ($ans eq 'syslogd'
+            or $ans eq 'rsyslogd'
             or $ans eq 'syslog-ng'
             or $ans eq 'ulogd'
             or $ans eq 'metalog'));
@@ -1821,45 +1842,26 @@ sub enable_psad_at_boot() {
     if (&query_yes_no("[+] Enable psad at boot time ([y]/n)?  ",
                 $ACCEPT_YES_DEFAULT)) {
         if ($distro eq 'redhat' or $distro eq 'fedora') {
-            system "$cmds{'chkconfig'} --add psad";
+            system "$cmds{'chkconfig'} --add $init_name";
         } elsif ($distro eq 'gentoo') {
-            system "$cmds{'rc-update'} add psad default";
-        } else {  ### it is a non-redhat distro, try to
-                  ### get the runlevel from /etc/inittab
-            if ($RUNLEVEL) {
-                ### the link already exists, so don't re-create it
-                unless (-e "/etc/rc.d/rc${RUNLEVEL}.d/S99psad") {
-                    symlink '/etc/rc.d/init.d/psad',
-                        "/etc/rc.d/rc${RUNLEVEL}.d/S99psad";
-                }
-            } elsif (-e '/etc/inittab') {
-                open I, '< /etc/inittab' or die "[*] Could not open ",
-                    "/etc/inittab: $!";
-                my @ilines = <I>;
-                close I;
-                for my $line (@ilines) {
-                    chomp $line;
-                    if ($line =~ /^id\:(\d)\:initdefault/) {
-                        $RUNLEVEL = $1;
-                        last;
+            system "$cmds{'rc-update'} add $init_name default";
+        } elsif ($distro eq 'ubuntu') {
+            system "$cmds{'update-rc.d'} $init_name defaults";
+        } else {
+
+            ### get the current run level
+            &get_runlevel();
+
+            if ($runlevel) {
+                if (-d '/etc/rc.d' and -d "/etc/rc.d/rc${runlevel}.d") {
+                    unless (-e "/etc/rc.d/rc${runlevel}.d/S99$init_name") {
+                        symlink "$init_dir/$init_name",
+                            "/etc/rc.d/rc${runlevel}.d/S99$init_name";
                     }
+                } else {
+                    print "[-] The /etc/rc.d/rc${runlevel}.d directory does ",
+                        "exist, not sure how to enable psad at boot time.";
                 }
-                unless ($RUNLEVEL) {
-                    &logr("[-] Could not determine the runlevel.  Set " .
-                        "the runlevel\nmanually in the config section of " .
-                        "install.pl\n");
-                    return;
-                }
-                ### the link already exists, so don't re-create it
-                unless (-e "/etc/rc.d/rc${RUNLEVEL}.d/S99psad") {
-                    symlink '/etc/rc.d/init.d/psad',
-                        "/etc/rc.d/rc${RUNLEVEL}.d/S99psad";
-                }
-            } else {
-                &logr("[-] /etc/inittab does not exist!  Set the " .
-                    "runlevel\nmanually in the config section of " .
-                    "install.pl.\n");
-                return;
             }
         }
     }
@@ -1880,6 +1882,14 @@ sub check_commands() {
                 }
             }
             unless ($found) {
+                if ($cmd eq 'runlevel') {
+                    if ($runlevel > 0) {
+                        next CMD;
+                    } else {
+                        die "[*] Could not find the $cmd command, ",
+                            "use --runlevel <N>";
+                    }
+                }
                 die "\n[*] Could not find $cmd anywhere!!!  ",
                     "Please edit the config section to include the path to ",
                     "$cmd.\n";
@@ -2028,6 +2038,25 @@ sub update_command_paths() {
     return;
 }
 
+sub get_runlevel() {
+    die "[*] The runlevel cannot be greater than 6"
+        if $runlevel > 6;
+
+    return if $runlevel > 0;
+
+    open RUN, "$cmds{'runlevel'} |" or die "[*] Could not execute the runlevel ",
+        "command, use --runlevel <N>";
+    while (<RUN>) {
+        if (/^\s*\S+\s+(\d+)/) {
+            $runlevel = $1;
+            last;
+        }
+    }
+    close RUN;
+    return;
+}
+
+
 ### logging subroutine that handles multiple filehandles
 sub logr() {
     my $msg = shift;
@@ -2068,10 +2097,22 @@ Usage: install.pl [options]
     -c, --config <file>          - Specify alternate path to psad.conf from
                                    which default installation paths are
                                    derived.
+    --init-dir <path>            - Specify path to the init directory (the
+                                   default is $init_dir).
+    --init-name <name>           - Specify the name for the psad init
+                                   script (the default is $init_name).
+    --install-syslog-fifo        - Add the installation of the psadfifo
+                                   (this is not usually necessary since
+                                   the default is to enable
+                                   ENABLE_SYSLOG_FILE).
+    -r, --runlevel <N>           - Specify the current system runlevel.
     --no-rm-lib-dir              - Do not remove the /usr/lib/psad/
                                    directory before installing psad.
     --no-syslog-test             - Skip syslog reconfiguration test.
     --no-preserve                - Disable preservation of old configs.
+    -L, --LANG <locale>          - Specify LANG env variable (actually the
+                                   LC_ALL variable).
+    -n, --no-LANG                - Do not export the LANG env variable.
     -h  --help                   - Prints this help message.
 
 _HELP_
