@@ -29,7 +29,7 @@
 *
 *****************************************************************************
 *
-*  $Id: psadwatchd.c 2047 2007-05-23 04:15:31Z mbr $
+*  $Id: psadwatchd.c 2136 2008-01-25 05:21:48Z mbr $
 */
 
 /* includes */
@@ -183,6 +183,7 @@ static void check_process(
     pid_t pid;
     unsigned short int restart = 0;
     char mail_str[MAX_MSG_LEN] = "";
+    char syslog_str[MAX_MSG_LEN] = "";
     char pid_line[MAX_PID_SIZE];
 
     if ((pidfile_ptr = fopen(pid_file, "r")) == NULL) {
@@ -236,6 +237,11 @@ static void check_process(
 #ifdef DEBUG
         fprintf(stderr, "sending mail:  %s\n", mail_str);
 #endif
+
+        snprintf(syslog_str, MAX_MSG_LEN,
+            "restarting %s on %s", pid_name, hostname);
+        slogr("psad(psadwatchd)", syslog_str);
+
         if (! no_email) {
             /* send the email */
             send_alert_email(shCmd, mailCmd, mail_str);
@@ -394,6 +400,7 @@ static void parse_config(void)
     int linectr = 0;
     char config_buf[MAX_LINE_BUF];
     char data_input_mode[MAX_GEN_LEN];
+    char enable_syslog_file[MAX_GEN_LEN];
     char *index;
 
     data_input_mode[0] = '\0';
@@ -429,6 +436,7 @@ static void parse_config(void)
             find_char_var("PSADWATCHD_MAX_RETRIES",
                 char_psadwatchd_max_retries, index);
             find_char_var("SYSLOG_DAEMON", data_input_mode, index);
+            find_char_var("ENABLE_SYSLOG_FILE", enable_syslog_file, index);
             find_char_var("EMAIL_ADDRESSES", mail_addrs, index);
 
             /* commands */
@@ -444,6 +452,11 @@ static void parse_config(void)
     if (strncmp(data_input_mode, "ulogd", MAX_GEN_LEN) == 0)
         check_kmsgsd = 0;
 
+    /* see if ENABLE_SYSLOG_FILE is enabled, so psad is just parsing
+     * a file written to by syslog directly */
+    if (strncmp(enable_syslog_file, "Y", 1) == 0)
+        check_kmsgsd = 0;
+
     /* resolve any embedded variables */
     expand_config_vars();
 
